@@ -13,7 +13,7 @@ from zeep.wsdl.definitions import AbstractOperation
 
 from python3_commons import object_storage
 from python3_commons.conf import S3Settings, s3_settings
-from python3_commons.object_storage import get_s3_client
+from python3_commons.object_storage import ObjectStorage
 
 logger = logging.getLogger(__name__)
 
@@ -24,16 +24,27 @@ class GeneratedStream(io.BytesIO):
         self.generator = generator
 
     def read(self, size: int = -1):
-        if size == -1:
-            size = 4096
+        if size < 0:
+            while True:
+                try:
+                    chunk = next(self.generator)
+                except StopIteration:
+                    break
+                else:
+                    self.write(chunk)
+        else:
+            total_written_size = 0
 
-        while self.tell() < size:
-            try:
-                chunk = next(self.generator)
-            except StopIteration:
-                break
+            while total_written_size < size:
+                try:
+                    chunk = next(self.generator)
+                except StopIteration:
+                    break
+                else:
+                    self.write(chunk)
+                    total_written_size += len(chunk)
 
-            self.write(chunk)
+        self.seek(0)
 
         if chunk := super().read(size):
             pos = self.tell()
@@ -44,6 +55,8 @@ class GeneratedStream(io.BytesIO):
             if unread_data_size > 0:
                 buf[:unread_data_size] = buf[pos:pos+unread_data_size]
 
+            del buf
+
             self.seek(0)
             self.truncate(unread_data_size)
 
@@ -82,7 +95,7 @@ def generate_archive(objects: Iterable[tuple[str, datetime, bytes]],
 def write_audit_data_sync(settings: S3Settings, key: str, data: bytes):
     if settings.s3_secret_access_key:
         try:
-            client = get_s3_client(settings)
+            client = ObjectStorage(settings).get_client()
             absolute_path = object_storage.get_absolute_path(f'audit/{key}')
 
             client.put_object(settings.s3_bucket, absolute_path, io.BytesIO(data), len(data))
